@@ -22,7 +22,7 @@ NNS.SSD <- function(x, y, plot = TRUE){
     if(any(class(x)%in%c("tbl","data.table"))) x <- as.vector(unlist(x))
     if(any(class(y)%in%c("tbl","data.table"))) y <- as.vector(unlist(y))
 
-    if(sum(is.na(cbind(x,y))) > 0) stop("You have some missing values, please address.")
+    if(anyNA(cbind(x,y))) stop("You have some missing values, please address.")
 
     Combined_sort <- sort(c(x, y), decreasing = FALSE)
 
